const webacdEnvironmentInput = document.getElementById('webacd-environment');
const appKeyInput = document.getElementById('appKey');
const clientCountInput = document.getElementById('client-count');
const originRoomIdInput = document.getElementById('origin-room-id');
const joinTimesInput = document.getElementById('join-times');
const talkingTimeInput = document.getElementById('talking-time');
const createClientTimeInput = document.getElementById('create-client-time');
const isUseVideoInput = document.getElementById('is-use-video');
const clearDataBtn = document.getElementById('clear-data');
const clientListInfo = document.getElementById('client-list-info');
const videoFile = document.getElementById('video-file');
const joinConAllBtn = document.getElementById('join-all');
const leaveConAllBtn = document.getElementById('leave-all');
const stopConAllBtn = document.getElementById('stop-all');
const uploadFileInput = document.getElementById('upload-file');
// const videoLevelInput = document.getElementById('video-level');
// const svcInput = document.getElementById('svc');
const openAllLocalStreamBtn = document.getElementById('open-all-local-stream');
const openAllRemoteStreamBtn = document.getElementById('open-all-remote-stream');


var {JuphoonRTCConference, Reporter, LoggerLevel, ConferenceEventType} = JuphoonRTCConferenceSDK;
var clientCount, originRoomId, joinCount, talkTime, useVideoFile, tokenList = [], getTokenList = [], clientList = [], intervalKeyList = [],
  loginIntervalList = [], termIntervalList = [];
var intervalKey = -1, termCurrentIndex = 0, createClientTime = 0, itemList = [], videoLevel, svc;

webacdEnvironmentInput.value = window.publicConfig.webacdEnvironment;
appKeyInput.value = window.publicConfig.appKey;

Reporter.setLogLevel(LoggerLevel.DEBUG);
JuphoonRTCConference.setConfig({address: webacdEnvironmentInput.value});

function startTest() {
  clientCount = parseInt(clientCountInput.value);
  originRoomId = originRoomIdInput.value;
  joinCount = parseInt(joinTimesInput.value);
  talkTime = parseInt(talkingTimeInput.value);
  createClientTime = parseInt(createClientTimeInput.value);
  useVideoFile = isUseVideoInput.checked;
  // videoLevel = parseInt(videoLevelInput.value);
  // svc = svcInput.value;
  if (useVideoFile) {
    videoFile.play();
  }
  bootstrapTest();
  joinConAllBtn.disabled = false;
  leaveConAllBtn.disabled = false;
  stopConAllBtn.disabled = false;
  openAllLocalStreamBtn.disabled = false;
  openAllRemoteStreamBtn.disabled = false;
}

function bootstrapTest() {
  let createdCount = 0, roomId = originRoomId;
  itemList.push({createdCount, joinCount: 0, roomId});
  createClient(createdCount)
    .then(client => {
      itemList[createdCount].client = client;
    });
  intervalKey = setInterval(() => {
      createdCount++;
      roomId++;
      if (createdCount >= clientCount) {
        clearInterval(intervalKey);
        return;
      }

      itemList.push({createdCount, joinCount: 0, roomId});
      createClient(createdCount)
        .then(client => {
          itemList[createdCount].client = client;
        });
    }, createClientTime * 1000);
}

// 创建client
function createClient(index) {
  const item = itemList[index];

  return new Promise((resolve, reject) => {
    JuphoonRTCConference.getToken()
      .then((_token) => {
        JuphoonRTCConference.config.socket = null;
        const client = JuphoonRTCConference.createConferenceClient(appKeyInput.value, _token);
        client.login()
          .then(() => {
            item.clientState.textContent = item.client.state;
            item.username.textContent = item.client.clientInfo.username;
            joinCon(index);
          });

        const wrapper = createView(index);
        item.outerBox = document.getElementById(`outer-box[${index}]`);
        item.localStream = document.getElementById(`local-stream[${index}]`);
        item.localStreamBtn = document.getElementById(`local-stream-btn[${index}]`);
        item.remoteStream = document.getElementById(`remote-stream[${index}]`);
        item.remoteStreamBtn = document.getElementById(`remote-stream-btn[${index}]`);
        item.username = document.getElementById(`username[${index}]`);
        item.groupNum = document.getElementById(`group-num`);
        item.clientState = document.getElementById(`client-state[${index}]`);
        item.socketState = document.getElementById(`socket-state[${index}]`);
        item.joinBtn = document.getElementById(`join-con[${index}]`);
        item.leaveBtn = document.getElementById(`leave-con[${index}]`);
        item.stopBtn = document.getElementById(`stop-con[${index}]`);

        client.addEventListener(ConferenceEventType.CLIENT_STATE_CHANGE, (ev) => {
          item.clientState.textContent = ev.message.state;
        });
        client.addEventListener(ConferenceEventType.SOCKET_STATE_CHANGE, (ev) => {
          item.socketState.textContent = ev.message.state;
        });
        client.addEventListener(ConferenceEventType.CONFERENCE_JOIN, (ev) => {
          if (!ev.message.result) return;

          client.startCameraVideo()
            .then(stream => {
              item.localRTCStream = stream;
              item.localStreamBtn.disabled = false;
            })
            .catch(reason => console.error(reason))
            .finally(() => {
              if (useVideoFile) {
                var stream = videoFile.captureStream();
                client.switchStream({
                  audio: false,
                  video: false,
                  videoSource: stream.getVideoTracks()[0],
                  audioSource: stream.getAudioTracks()[0]
                });
              }
            });
          client.startVideo()
            .then(stream => {
              item.remoteRTCStream = stream;
              item.remoteStreamBtn.disabled = false;
            })
            .catch(err => {
              console.error(err.type);
            });
          setTimeout(() => leaveCon(index), talkTime * 1000);
        });

        resolve(client);
      })
      .catch(reason => reject(reason));
  });
}

function createView(index) {
  const outerBox = document.createElement('div');
  outerBox.classList.add('mdui-card', 'card-box');
  outerBox.id = `outer-box[${index}]`;
  outerBox.innerHTML = `
    <div class="video-block" id="local-stream[${index}]">
      <button class="mdui-btn mdui-color-green change-btn float-btn" id="local-stream-btn[${index}]" onclick="openLocalStream(${index})" disabled>本地画面</button>
    </div>
    <div class="video-block" id="remote-stream[${index}]">
      <button class="mdui-btn mdui-color-green change-btn float-btn" id="remote-stream-btn[${index}]" onclick="openRemoteStream(${index})" disabled>远端画面</button>
    </div>
    <ul class="left-li-block">
      <li>
        <div class="ul-li-span"><b>username: </b><span id="username[${index}]">null</span></div>
        <div class="ul-li-span"><b>room id: </b><span id="group-num">${itemList[index].roomId}</span></div>
      </li>
      <li><div class="ul-li-span"><b>client状态: </b><span id="client-state[${index}]"></span></div>
        <div class="ul-li-span"><b>socket状态: </b><span id="socket-state[${index}]">null</span></div>
      </li>
    </ul>
    <div class="btn-block">
      <button class="mdui-btn mdui-color-theme-accent change-btn" disabled id="join-con[${index}]" onclick="joinCon(${index})">加入</button>
      <button class="mdui-btn mdui-color-red change-btn" disabled id="leave-con[${index}]" onclick="leaveCon(${index})">离开</button>
      <button class="mdui-btn mdui-color-red change-btn" disabled id="stop-con[${index}]" onclick="stopCon(${index})">结束</button>
      <ul class="left-li-block">
        <li>
        </li>
      </ul>
    </div>
    `;
  clientListInfo.append(outerBox);

  return outerBox;
}

// 加入会议
function joinCon(index) {
  const item = itemList[index];
  const client = itemList[index].client;

  item.joinBtn.disabled = true;
  client.join(item.roomId.toString(), true)
    .then(() => {
    })
    .catch((reason) => {
      console.log('加入会议失败，请检查参数', reason);
    });
  client.addEventListener(ConferenceEventType.CONFERENCE_LEAVE, (ev) => {
    item.clientState.textContent = client.state;
  });
  client.addEventListener(ConferenceEventType.CONFERENCE_DESTROY, (ev) => {
    item.clientState.textContent = client.state;
  });
}

// 离开会议
function leaveCon(index) {
  const item = itemList[index];
  const client = itemList[index].client;

  item.leaveBtn.disabled = true;
  client.leave();
  client._remoteStream && client._remoteStream.stop();
  client._localStream && client._localStream.stop();
}

// 开启本地视频
function openLocalStream(index) {
  const item = itemList[index];
  const client = itemList[index].client;

  item.localStreamBtn.disabled = true;
  item.localRTCStream.start(`local-stream[${index}]`)
}

// 开启远端视频
function openRemoteStream(index) {
  const item = itemList[index];
  const client = itemList[index].client;

  item.remoteStreamBtn.disabled = true;
  item.remoteRTCStream.start(`remote-stream[${index}]`)
    .then(dom => {
      dom.muted = false;
    });
}

// 全部加入
function joinConAll() {
  joinConAllBtn.disabled = true;
  for (let i = 0; i < itemList.length; i++) {
    joinCon(i);
  }
}

// 全部离开
function leaveConAll() {
  leaveConAllBtn.disabled = true;
  for (let i = 0; i < itemList.length; i++) {
    leaveCon(i);
  }
}

// 全部结束
function stopConAll() {
  stopConAllBtn.disabled = true;
  for (let i = 0; i < itemList.length; i++) {
    stopCon(i);
  }
}

//开启全部本地视频
function openAllLocalStream() {
  openAllLocalStreamBtn.disabled = true;
  for (let i = 0; i < itemList.length; i++) {
    openLocalStream(i);
  }
}

//开启全部远端视频
function openAllRemoteStream() {
  openAllRemoteStreamBtn.disabled = true;
  for (let i = 0; i < itemList.length; i++) {
    openRemoteStream(i);
  }
}

// 上传本地视频文件
function switchVideoFileStream(files) {
  const URL = window.URL || window.webkitURL;
  const videoUrl = URL.createObjectURL(files[0]);
  videoFile.src = videoUrl;
  videoFile.play();

  setAllClientVideoFileStream(videoFile);
}

// 清空数据
function resetData() {
  for (let i = itemList.length; i--;) {
    itemList[i].client.destroy();
  }
  itemList = [];
  clientListInfo.innerHTML = '';
  clearInterval(intervalKey);
}

// function videoLevelChange() {
//   setAllClientVideolevel(videoLevelInput.value);
// }

// function setAllClientVideolevel(videoLevel) {
//   videoLevel = parseInt(videoLevel);
//
//   for (let i = itemList.length; i--;) {
//     itemList[i].client && itemList[i].client.setRequestVideo(videoLevel);
//   }
// }

function setAllClientVideoFileStream(videoDOM) {
  for (let i = itemList.length; i--;) {
    setClientVideoFileStream(i, videoDOM);
  }
}

function setClientVideoFileStream(index, videoDOM) {
  const client = itemList[index].client;
  try {
    const stream = videoDOM.captureStream();
    client.switchStream({
      audio: false,
      video: false,
      audioSource: stream.getAudioTracks()[0],
      videoSource: stream.getVideoTracks()[0]
    });
  } catch (e) {
    console.error(e);
  }
}
